# generated by datamodel-codegen:
#   filename:  media_directories.json

from __future__ import annotations

from pydantic import BaseModel


class Directory(BaseModel):
    key: str
    path: str


class MediaDirectories(BaseModel):
    directories: list[Directory]



# Data

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field

from .battery import Battery
from .cpu import Cpu
from .disk import Disk
from .display import Display
from .gpu import Gpu
from .memory import Memory
from .network import Network
from .sensors import Sensors
from .system import System


class Data(BaseModel):
    """Data"""

    battery: Battery = Field(..., alias="battery")
    cpu: Cpu = Field(..., alias="cpu")
    disk: Disk = Field(..., alias="disk")
    display: Display = Field(..., alias="display")
    gpu: Gpu = Field(..., alias="gpu")
    memory: Memory = Field(..., alias="memory")
    network: Network = Field(..., alias="network")
    sensors: Sensors = Field(..., alias="sensors")
    system: System = Field(..., alias="system")


class DataDict(BaseModel):
    class Config:
        extra = Extra.allow

    last_updated: dict[str, Optional[float]] = Field(..., description="Last updated")



"""System Bridge: Settings"""
from __future__ import annotations

import io
import os
from os.path import exists
from typing import Any, Union
from uuid import uuid4

from appdirs import AppDirs
from cryptography.fernet import Fernet

from .base import Base
from .common import convert_string_to_correct_type
from .const import (
    SECRET_API_KEY,
    SETTING_ADDITIONAL_MEDIA_DIRECTORIES,
    SETTING_AUTOSTART,
    SETTING_KEYBOARD_HOTKEYS,
    SETTING_LOG_LEVEL,
    SETTING_PORT_API,
)
from .database import Database
from .models.database_data import Data as DatabaseData
from .models.database_data import Secrets as DatabaseSecrets
from .models.database_data import Settings as DatabaseSettings


class Settings(Base):
    """Settings"""

    def __init__(
        self,
        database: Database,
    ) -> None:
        """Initialize"""
        super().__init__()
        self._database = database

        # Generate default encryption key
        self._encryption_key: str = ""
        secret_key_path = os.path.join(
            AppDirs("systembridge", "timmo001").user_data_dir, "secret.key"
        )
        if exists(secret_key_path):
            with io.open(secret_key_path, encoding="utf-8") as file:
                self._encryption_key = file.read().splitlines()[0]
        if not self._encryption_key:
            self._encryption_key = Fernet.generate_key().decode()
            with io.open(secret_key_path, "w", encoding="utf-8") as file:
                file.write(self._encryption_key)

        # Default Secrets
        if self._database.get_data_item_by_key(DatabaseSecrets, SECRET_API_KEY) is None:
            self.set_secret(SECRET_API_KEY, str(uuid4()))

        # Default Settings
        if (
            self._database.get_data_item_by_key(DatabaseSettings, SETTING_AUTOSTART)
            is None
        ):
            self.set(SETTING_AUTOSTART, str(False))
        if (
            self._database.get_data_item_by_key(DatabaseSettings, SETTING_LOG_LEVEL)
            is None
        ):
            self.set(SETTING_LOG_LEVEL, "INFO")
        if (
            self._database.get_data_item_by_key(DatabaseSettings, SETTING_PORT_API)
            is None
        ):
            self.set(SETTING_PORT_API, str(9170))
        if (
            self._database.get_data_item_by_key(
                DatabaseSettings, SETTING_ADDITIONAL_MEDIA_DIRECTORIES
            )
            is None
        ):
            self.set(SETTING_ADDITIONAL_MEDIA_DIRECTORIES, str([]))
        if (
            self._database.get_data_item_by_key(
                DatabaseSettings, SETTING_KEYBOARD_HOTKEYS
            )
            is None
        ):
            self.set(SETTING_KEYBOARD_HOTKEYS, str([]))

    def get_all(self) -> list[DatabaseData]:
        """Get settings"""
        records = self._database.get_data(DatabaseSettings)
        for record in records:
            if record.value is not None:
                record.value = convert_string_to_correct_type(record.value)
        return records

    def get(
        self,
        key: str,
    ) -> Union[bool, float, int, str, list[Any], dict[str, Any], None]:
        """Get setting"""
        record = self._database.get_data_item_by_key(DatabaseSettings, key)
        if record is None or record.value is None:
            return None
        return convert_string_to_correct_type(record.value)

    def get_secret(
        self,
        key: str,
    ) -> str:
        """Get secret"""
        record = self._database.get_data_item_by_key(DatabaseSecrets, key)
        if record is None or record.value is None:
            raise KeyError(f"Secret {key} not found")

        secret = record.value
        fernet = Fernet(self._encryption_key)
        return fernet.decrypt(secret.encode()).decode()

    def set(
        self,
        key: str,
        value: str,
    ) -> None:
        """Set setting"""
        self._database.update_data(
            DatabaseSettings,
            DatabaseSettings(
                key=key,
                value=value,
            ),
        )

    def set_secret(
        self,
        key: str,
        value: str,
    ) -> None:
        """Set secret"""
        fernet = Fernet(self._encryption_key)

        self._database.update_data(
            DatabaseSecrets,
            DatabaseSecrets(
                key=key,
                value=fernet.encrypt(value.encode()).decode(),
            ),
        )



# generated by datamodel-codegen:
#   filename:  open_url.json

from __future__ import annotations

from pydantic import BaseModel


class OpenUrl(BaseModel):
    """
    Open URL
    """

    url: str



# generated by datamodel-codegen:
#   filename:  disk.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    devices: float
    io_counters_read_count: Optional[float] = None
    io_counters_write_count: Optional[float] = None
    io_counters_read_bytes: Optional[float] = None
    io_counters_write_bytes: Optional[float] = None
    io_counters_read_time: Optional[float] = None
    io_counters_write_time: Optional[float] = None
    partitions: float


class Disk(BaseModel):
    """
    Disk
    """

    class Config:
        extra = Extra.allow

    id: Optional[str] = Field(None, description="Event ID")
    devices: list
    io_counters_read_count: Optional[int] = None
    io_counters_write_count: Optional[int] = None
    io_counters_read_bytes: Optional[int] = None
    io_counters_write_bytes: Optional[int] = None
    io_counters_read_time: Optional[int] = None
    io_counters_write_time: Optional[int] = None
    partitions: list
    last_updated: Optional[LastUpdated] = Field(None, description="Last updated")



"""System Bridge: Update"""
from __future__ import annotations

import importlib.util
import os
import platform
import subprocess
import sys
import urllib.request
from json import loads
from typing import Optional

from .base import Base
from .common import asyncio_get_loop


class Update(Base):
    """Update"""

    def _install_package(
        self,
        package: str,
    ) -> None:
        """Install a package."""
        args = [
            sys.executable,
            "-m",
            "pip",
            "install",
            package,
        ]
        self._logger.debug(" ".join(args))
        with subprocess.Popen(args, stdout=subprocess.PIPE) as pipe:
            result = pipe.communicate()[0].decode()
        self._logger.info("Result: %s", result)

    def _update(self, packages: dict[str, Optional[str]]) -> None:
        """Update each package."""
        for package, version in packages.items():
            self._logger.info("Updating %s to version %s", package, version)
            if version:
                self._install_package(f"{package}=={version}")

        asyncio_get_loop().call_later(
            5, os.execl, sys.executable, sys.executable, *sys.argv
        )

    def update(
        self,
        version: Optional[str],
        wait: bool = False,
    ) -> dict[str, Optional[str]]:
        """Update the application."""
        packages = {
            "systembridgeshared": version,
            "systembridgebackend": version,
        }
        if importlib.util.find_spec("systembridgecli") is not None:
            packages["systembridgecli"] = version
        if importlib.util.find_spec("systembridgeconnector") is not None:
            packages["systembridgeconnector"] = version
        if importlib.util.find_spec("systembridgefrontend") is not None:
            packages["systembridgefrontend"] = version
        if importlib.util.find_spec("systembridgegui") is not None:
            packages["systembridgegui"] = version
        if (
            "Windows" in platform.system()
            and importlib.util.find_spec("systembridgewindowssensors") is not None
        ):
            packages["systembridgewindowssensors"] = version

        for package, new_version in packages.items():
            if new_version is None:
                with urllib.request.urlopen(
                    f"https://pypi.org/pypi/{package}/json"
                ) as response:
                    data = loads(response.read())
                new_version = data["info"]["version"]
                packages[package] = new_version

        if wait:
            self._update(packages)
        else:
            asyncio_get_loop().call_later(2, self._update, packages)

        return packages



"""
Provides systembridgeshared version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update systembridgeshared` to change this file.

from incremental import Version

__version__ = Version("systembridgeshared", 3, 6, 3, dev=1)
__all__ = ["__version__"]



# generated by datamodel-codegen:
#   filename:  media_files.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class File(BaseModel):
    """
    Media File
    """

    name: str
    path: str
    fullpath: str
    size: int
    last_accessed: float
    created: float
    modified: float
    is_directory: bool
    is_file: bool
    is_link: bool
    mime_type: Optional[str] = None


class MediaFiles(BaseModel):
    """
    Media Files
    """

    files: list[File]
    path: str



# generated by datamodel-codegen:
#   filename:  notification.json

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel, confloat


class Action(BaseModel):
    """
    Notification Action
    """

    command: str
    data: Optional[dict[str, Any]] = None
    label: str


class Audio(BaseModel):
    source: str
    volume: Optional[confloat(ge=0.0, le=100.0)] = None


class Notification(BaseModel):
    """
    Notification
    """

    title: str
    message: Optional[str] = None
    icon: Optional[str] = None
    image: Optional[str] = None
    actions: Optional[list[Action]] = None
    timeout: Optional[float] = None
    audio: Optional[Audio] = None



# generated by datamodel-codegen:
#   filename:  cpu.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    count: Optional[float] = None
    frequency_current: Optional[float] = None
    frequency_min: Optional[float] = None
    frequency_max: Optional[float] = None
    load_average: Optional[float] = None
    power_package: Optional[float] = None
    stats_ctx_switches: Optional[float] = None
    stats_interrupts: Optional[float] = None
    stats_soft_interrupts: Optional[float] = None
    stats_syscalls: Optional[float] = None
    temperature: Optional[float] = None
    times_user: Optional[float] = None
    times_system: Optional[float] = None
    times_idle: Optional[float] = None
    times_interrupt: Optional[float] = None
    times_dpc: Optional[float] = None
    times_percent_user: Optional[float] = None
    times_percent_system: Optional[float] = None
    times_percent_idle: Optional[float] = None
    times_percent_interrupt: Optional[float] = None
    times_percent_dpc: Optional[float] = None
    usage: Optional[float] = None
    voltage: Optional[float] = None


class Cpu(BaseModel):
    """
    CPU
    """

    class Config:
        extra = Extra.allow

    id: Optional[str] = Field(None, description="Event ID")
    count: Optional[int] = None
    frequency_current: Optional[float] = None
    frequency_min: Optional[float] = None
    frequency_max: Optional[float] = None
    load_average: Optional[float] = None
    power_package: Optional[float] = None
    stats_ctx_switches: Optional[int] = None
    stats_interrupts: Optional[int] = None
    stats_soft_interrupts: Optional[int] = None
    stats_syscalls: Optional[int] = None
    temperature: Optional[float] = None
    times_user: Optional[float] = None
    times_system: Optional[float] = None
    times_idle: Optional[float] = None
    times_interrupt: Optional[float] = None
    times_dpc: Optional[float] = None
    times_percent_user: Optional[float] = None
    times_percent_system: Optional[float] = None
    times_percent_idle: Optional[float] = None
    times_percent_interrupt: Optional[float] = None
    times_percent_dpc: Optional[float] = None
    usage: Optional[float] = None
    voltage: Optional[float] = None
    last_updated: Optional[LastUpdated] = Field(None, description="Last updated")



"""System Bridge Shared: Common"""
from __future__ import annotations

import asyncio
import json
import os
import re
from typing import Any, Union

from appdirs import AppDirs


def get_user_data_directory() -> str:
    """Get the user data directory"""
    user_data_dir = AppDirs("systembridge", "timmo001").user_data_dir
    # Create User Data Directories
    if not os.path.exists(user_data_dir):
        os.makedirs(user_data_dir, exist_ok=True)
    return user_data_dir


def camel_to_snake(name):
    """Convert camel case to snake case"""
    name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", name).lower()


def convert_string_to_correct_type(
    value: str | None,
) -> Union[bool, float, int, str, list[Any], dict[str, Any], None]:
    """Convert string to correct data type"""
    if value is None:
        return None
    try:
        if value.startswith("'") and value.endswith("'"):
            return convert_string_to_correct_type(value[1:-1])
        if (value.startswith("[") and value.endswith("]")) or (
            value.startswith("{") and value.endswith("}")
        ):
            try:
                return json.loads(value)
            except json.JSONDecodeError:
                return value
        value_lower = value.lower()
        if value_lower in ("none", "null", "nan"):
            return None
        if value_lower == "true":
            return True
        if value_lower == "false":
            return False
        if value.isalpha():
            return value
        if value.isdigit():
            return int(value)
        return float(value)
    except ValueError:
        return value


def make_key(key_input: str) -> str:
    """Make a key from a string"""
    return (
        key_input.replace(" ", "_", -1)
        .replace("(", "", -1)
        .replace(")", "", -1)
        .replace("\\", "", -1)
        .replace(".", "", -1)
        .lower()
    )


def asyncio_get_loop() -> asyncio.AbstractEventLoop:
    """Get the event loop"""
    try:
        loop = asyncio.get_event_loop()
        if not loop.is_running():
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
        return loop
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        return loop



# generated by datamodel-codegen:
#   filename:  action.json

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class Action(BaseModel):
    """
    Action
    """

    command: str
    data: Optional[dict[str, Any]] = None
    label: Optional[str] = None



# generated by datamodel-codegen:
#   filename:  media_get_files.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class MediaGetFiles(BaseModel):
    """
    Get Files
    """

    base: str
    path: Optional[str] = None



# generated by datamodel-codegen:
#   filename:  network.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    io_counters_bytes_sent: Optional[float] = None
    io_counters_bytes_recv: Optional[float] = None
    io_counters_packets_sent: Optional[float] = None
    io_counters_packets_recv: Optional[float] = None
    io_counters_errin: Optional[float] = None
    io_counters_errout: Optional[float] = None
    io_counters_dropin: Optional[float] = None
    io_counters_dropout: Optional[float] = None


class Network(BaseModel):
    """
    Network
    """

    class Config:
        extra = Extra.allow

    id: Optional[str] = Field(None, description="Event ID")
    io_counters_bytes_sent: Optional[int] = None
    io_counters_bytes_recv: Optional[int] = None
    io_counters_packets_sent: Optional[int] = None
    io_counters_packets_recv: Optional[int] = None
    io_counters_errin: Optional[int] = None
    io_counters_errout: Optional[int] = None
    io_counters_dropin: Optional[int] = None
    io_counters_dropout: Optional[int] = None
    last_updated: Optional[LastUpdated] = Field(None, description="Last updated")



"""System Bridge Shared: HTTP Client"""
from __future__ import annotations

import asyncio
from typing import Any, Optional

import async_timeout
from aiohttp import ClientResponse, ClientSession
from aiohttp.client_exceptions import ClientConnectorError, ServerDisconnectedError

from .base import Base
from .exceptions import AuthenticationException, ConnectionErrorException

BASE_HEADERS = {
    "Accept": "application/json",
}


class HTTPClient(Base):
    """Client to handle API calls."""

    def __init__(
        self,
        api_host: str,
        api_port: int,
        api_key: str,
        session: Optional[ClientSession] = None,
    ) -> None:
        """Initialize the client."""
        super().__init__()
        self._api_key = api_key
        self._base_url = f"http://{api_host}:{api_port}"
        self._session = session if session else ClientSession()

    async def delete(
        self,
        path: str,
        payload: Optional[Any],
    ) -> Any:
        """Make a DELETE request"""
        response: ClientResponse = await self.request(
            "DELETE",
            f"{self._base_url}{path}",
            headers={
                **BASE_HEADERS,
                "api-key": self._api_key,
            },
            json=payload,
        )
        return await response.json()

    async def get(
        self,
        path: str,
    ) -> Any:
        """Make a GET request"""
        response: ClientResponse = await self.request(
            "GET",
            f"{self._base_url}{path}",
            headers={
                **BASE_HEADERS,
                "api-key": self._api_key,
            },
        )
        if "application/json" in response.headers.get("Content-Type", ""):
            return await response.json()
        return await response.text()

    async def post(
        self,
        path: str,
        payload: Optional[Any],
    ) -> Any:
        """Make a POST request"""
        response: ClientResponse = await self.request(
            "POST",
            f"{self._base_url}{path}",
            headers={
                **BASE_HEADERS,
                "api-key": self._api_key,
            },
            json=payload,
        )
        return await response.json()

    async def put(
        self,
        path: str,
        payload: Optional[Any],
    ) -> Any:
        """Make a PUT request"""
        response: ClientResponse = await self.request(
            "PUT",
            f"{self._base_url}{path}",
            headers={
                **BASE_HEADERS,
                "api-key": self._api_key,
            },
            json=payload,
        )
        return await response.json()

    async def request(
        self,
        method: str,
        url: str,
        **kwargs,
    ) -> ClientResponse:
        """Make a request."""
        try:
            async with async_timeout.timeout(20):
                response: ClientResponse = await self._session.request(
                    method,
                    url,
                    **kwargs,
                )
            if response.status not in (200, 201, 202, 204):
                if response.status in (401, 403):
                    raise AuthenticationException(
                        {
                            "request": {
                                "method": method,
                                "url": url,
                            },
                            "response": await response.json(),
                            "status": response.status,
                        }
                    )
                raise ConnectionErrorException(
                    {
                        "request": {
                            "method": method,
                            "url": url,
                        },
                        "response": await response.json(),
                        "status": response.status,
                    }
                )
            return response
        except asyncio.TimeoutError as exception:
            raise ConnectionErrorException(
                {
                    "request": {
                        "method": method,
                        "url": url,
                    },
                    "status": "timeout",
                }
            ) from exception
        except (
            ClientConnectorError,
            ConnectionResetError,
            ServerDisconnectedError,
        ) as exception:
            raise ConnectionErrorException(
                {
                    "request": {
                        "method": method,
                        "url": url,
                    },
                    "status": "connection error",
                }
            ) from exception



"""System Bridge Shared: WebSocket Client"""
from __future__ import annotations

import asyncio
import socket
from collections.abc import Awaitable, Callable
from typing import Any, Optional
from uuid import uuid4

import aiohttp

from .base import Base
from .const import (
    EVENT_API_KEY,
    EVENT_DATA,
    EVENT_EVENT,
    EVENT_ID,
    EVENT_MESSAGE,
    EVENT_MODULE,
    EVENT_SUBTYPE,
    EVENT_TYPE,
    MODEL_MAP,
    MODEL_RESPONSE,
    SECRET_API_KEY,
    SETTING_PORT_API,
    SUBTYPE_BAD_API_KEY,
    SUBTYPE_LISTENER_ALREADY_REGISTERED,
    TYPE_APPLICATION_UPDATE,
    TYPE_DATA_UPDATE,
    TYPE_ERROR,
    TYPE_EXIT_APPLICATION,
    TYPE_GET_DATA,
    TYPE_GET_DIRECTORIES,
    TYPE_GET_FILE,
    TYPE_GET_FILES,
    TYPE_KEYBOARD_KEYPRESS,
    TYPE_KEYBOARD_TEXT,
    TYPE_NOTIFICATION,
    TYPE_OPEN,
    TYPE_POWER_HIBERNATE,
    TYPE_POWER_LOCK,
    TYPE_POWER_LOGOUT,
    TYPE_POWER_RESTART,
    TYPE_POWER_SHUTDOWN,
    TYPE_POWER_SLEEP,
    TYPE_REGISTER_DATA_LISTENER,
)
from .exceptions import (
    AuthenticationException,
    BadMessageException,
    ConnectionClosedException,
    ConnectionErrorException,
)
from .models.get_data import GetData
from .models.keyboard_key import KeyboardKey
from .models.keyboard_text import KeyboardText
from .models.media_directories import MediaDirectories
from .models.media_files import File as MediaFile
from .models.media_files import MediaFiles
from .models.media_get_file import MediaGetFile
from .models.media_get_files import MediaGetFiles
from .models.notification import Notification
from .models.open_path import OpenPath
from .models.open_url import OpenUrl
from .models.register_data_listener import RegisterDataListener
from .models.request import Request
from .models.response import Response
from .models.update import Update
from .settings import Settings


class WebSocketClient(Base):
    """WebSocket Client"""

    def __init__(
        self,
        settings: Settings,
    ) -> None:
        """Initialize"""
        super().__init__()
        self._settings = settings
        self._responses: dict[str, tuple[asyncio.Future[Response], Optional[str]]] = {}
        self._session: Optional[aiohttp.ClientSession] = None
        self._websocket: Optional[aiohttp.ClientWebSocketResponse] = None
        self._api_key = self._settings.get_secret(SECRET_API_KEY)

    @property
    def connected(self) -> bool:
        """Get connection state."""
        return self._websocket is not None and not self._websocket.closed

    async def _send_message(
        self,
        request: Request,
        wait_for_response: bool = True,
        response_type: Optional[str] = None,
    ) -> Response:
        """Send a message to the WebSocket"""
        if not self.connected or self._websocket is None:
            raise ConnectionClosedException("Connection is closed")

        request.api_key = self._api_key
        request.id = uuid4().hex
        future: asyncio.Future[Response] = asyncio.get_running_loop().create_future()
        self._responses[request.id] = future, response_type
        await self._websocket.send_str(request.json())
        self._logger.debug("Sent message: %s", request.json(exclude={EVENT_API_KEY}))
        if wait_for_response:
            try:
                return await future
            finally:
                self._responses.pop(request.id)
        return Response(
            id=request.id,
            type="N/A",
            message="Message sent",
            subtype=None,
            module=None,
            data=None,
        )

    async def close(self) -> None:
        """Close connection"""
        self._logger.info("Closing WebSocket connection")
        if self._websocket is not None:
            await self._websocket.close()
        if self._session is not None:
            await self._session.close()

    async def connect(
        self,
        session: Optional[aiohttp.ClientSession] = None,
    ) -> None:
        """Connect to server"""
        if session:
            self._session = session
        else:
            self._logger.info("Creating new aiohttp client session")
            self._session = aiohttp.ClientSession()
        url = f"ws://localhost:{self._settings.get(SETTING_PORT_API)}/api/websocket"
        self._logger.info(
            "Connecting to WebSocket: %s (aiohttp: %s)",
            url,
            aiohttp.__version__,
        )
        try:
            self._websocket = await self._session.ws_connect(url=url, heartbeat=30)
        except (
            aiohttp.WSServerHandshakeError,
            aiohttp.ClientConnectionError,
            socket.gaierror,
        ) as error:
            self._logger.warning(
                "Failed to connect to WebSocket: %s - %s",
                error.__class__.__name__,
                error,
            )
            raise ConnectionErrorException from error
        self._logger.info("Connected to WebSocket")

    async def application_update(
        self,
        model: Update,
    ) -> Response:
        """Update application"""
        self._logger.info("Updating application")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_APPLICATION_UPDATE,
                    **model.dict(),
                }
            ),
            wait_for_response=False,
        )

    async def exit_backend(self) -> Response:
        """Exit backend"""
        self._logger.info("Exiting backend")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_EXIT_APPLICATION,
                }
            ),
            wait_for_response=False,
        )

    async def get_data(
        self,
        model: GetData,
    ) -> Response:
        """Get data from server"""
        self._logger.info("Getting data from server: %s", model.json())
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_GET_DATA,
                    **model.dict(),
                }
            ),
            response_type=TYPE_DATA_UPDATE,
        )

    async def get_directories(self) -> MediaDirectories:
        """Get directories"""
        self._logger.info("Getting directories:")
        response = await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_GET_DIRECTORIES,
                }
            )
        )
        return MediaDirectories(**response.dict())

    async def get_files(
        self,
        model: MediaGetFiles,
    ) -> MediaFiles:
        """Get files"""
        self._logger.info("Getting files: %s", model.json())
        response = await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_GET_FILES,
                    **model.dict(),
                }
            )
        )
        return MediaFiles(**response.dict())

    async def get_file(
        self,
        model: MediaGetFile,
    ) -> MediaFile:
        """Get files"""
        self._logger.info("Getting file: %s", model.json())
        response = await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_GET_FILE,
                    **model.dict(),
                }
            )
        )
        return MediaFile(**response.dict())

    async def register_data_listener(
        self,
        model: RegisterDataListener,
    ) -> Response:
        """Register data listener"""
        self._logger.info("Registering data listener: %s", model.json())
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_REGISTER_DATA_LISTENER,
                    **model.dict(),
                }
            )
        )

    async def keyboard_keypress(
        self,
        model: KeyboardKey,
    ) -> Response:
        """Keyboard keypress"""
        self._logger.info("Press key: %s", model.json())
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_KEYBOARD_KEYPRESS,
                    **model.dict(),
                }
            )
        )

    async def keyboard_text(
        self,
        model: KeyboardText,
    ) -> Response:
        """Keyboard keypress"""
        self._logger.info("Enter text: %s", model.json())
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_KEYBOARD_TEXT,
                    **model.dict(),
                }
            )
        )

    async def send_notification(
        self,
        model: Notification,
    ) -> Response:
        """Send notification"""
        self._logger.info("Send notification: %s", model.json())
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_NOTIFICATION,
                    **model.dict(),
                }
            )
        )

    async def open_path(
        self,
        model: OpenPath,
    ) -> Response:
        """Open path"""
        self._logger.info("Opening path: %s", model.json())
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_OPEN,
                    **model.dict(),
                }
            )
        )

    async def open_url(
        self,
        model: OpenUrl,
    ) -> Response:
        """Open url"""
        self._logger.info("Opening URL: %s", model.json())
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_OPEN,
                    **model.dict(),
                }
            )
        )

    async def power_sleep(self) -> Response:
        """Power sleep"""
        self._logger.info("Power sleep")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_POWER_SLEEP,
                }
            )
        )

    async def power_hibernate(self) -> Response:
        """Power hibernate"""
        self._logger.info("Power hibernate")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_POWER_HIBERNATE,
                }
            )
        )

    async def power_restart(self) -> Response:
        """Power restart"""
        self._logger.info("Power restart")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_POWER_RESTART,
                }
            )
        )

    async def power_shutdown(self) -> Response:
        """Power shutdown"""
        self._logger.info("Power shutdown")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_POWER_SHUTDOWN,
                }
            )
        )

    async def power_lock(self) -> Response:
        """Power lock"""
        self._logger.info("Power lock")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_POWER_LOCK,
                }
            )
        )

    async def power_logout(self) -> Response:
        """Power logout"""
        self._logger.info("Power logout")
        return await self._send_message(
            Request(
                **{
                    EVENT_EVENT: TYPE_POWER_LOGOUT,
                }
            )
        )

    async def listen(
        self,
        callback: Optional[Callable[[str, Any], Awaitable[None]]] = None,
        accept_other_types: bool = False,
    ) -> None:
        """Listen for messages and map to modules"""

        async def _callback_message(message: dict) -> None:
            """Message Callback"""
            self._logger.debug("New message: %s", message[EVENT_TYPE])

            if message.get(EVENT_ID) is not None:
                response_tuple = self._responses.get(message[EVENT_ID])
                if response_tuple is not None:
                    future, response_type = response_tuple
                    if (
                        response_type is not None
                        and response_type != message[EVENT_TYPE]
                    ):
                        self._logger.info(
                            "Response type '%s' does not match requested type '%s'.",
                            message[EVENT_TYPE],
                            response_type,
                        )
                    else:
                        response = Response(**message)

                        if (
                            response.type == TYPE_DATA_UPDATE
                            and response.module is not None
                            and message[EVENT_DATA] is not None
                        ):
                            # Find model from module
                            model = MODEL_MAP.get(message[EVENT_MODULE])
                            if model is None:
                                self._logger.warning(
                                    "Unknown model: %s", message[EVENT_MODULE]
                                )
                            else:
                                response.data = model(**message[EVENT_DATA])

                        self._logger.info(
                            "Response: %s",
                            response.json(
                                include={
                                    EVENT_ID,
                                    EVENT_TYPE,
                                    EVENT_SUBTYPE,
                                    EVENT_MESSAGE,
                                },
                                exclude_unset=True,
                            ),
                        )

                        try:
                            future.set_result(response)
                        except asyncio.InvalidStateError:
                            self._logger.warning(
                                "Future already set for response ID: %s",
                                message[EVENT_ID],
                            )

            if message[EVENT_TYPE] == TYPE_ERROR:
                if message[EVENT_SUBTYPE] == SUBTYPE_LISTENER_ALREADY_REGISTERED:
                    self._logger.debug(message)
                else:
                    self._logger.warning("Error message: %s", message)
            elif (
                message[EVENT_TYPE] == TYPE_DATA_UPDATE
                and message[EVENT_DATA] is not None
            ):
                self._logger.debug(
                    "New data for: %s\n%s", message[EVENT_MODULE], message[EVENT_DATA]
                )
                model = MODEL_MAP.get(message[EVENT_MODULE])
                if model is None:
                    self._logger.warning("Unknown model: %s", message[EVENT_MODULE])
                else:
                    if callback is not None:
                        await callback(
                            message[EVENT_MODULE],
                            model(**message[EVENT_DATA]),
                        )
            else:
                self._logger.debug("Other message: %s", message[EVENT_TYPE])
                if accept_other_types:
                    model = MODEL_MAP.get(EVENT_TYPE, MODEL_MAP[MODEL_RESPONSE])
                    if model is not None and callback is not None:
                        await callback(
                            message[EVENT_TYPE],
                            model(**message),
                        )

        await self.listen_for_messages(callback=_callback_message)

    async def listen_for_messages(
        self,
        callback: Callable,
    ) -> None:
        """Listen for messages"""
        if not self.connected:
            raise ConnectionClosedException("Connection is closed")

        self._logger.info("Listen for messages")
        if self._websocket is not None:
            while not self._websocket.closed:
                message = await self.receive_message()
                if isinstance(message, dict):
                    await callback(message)

    async def receive_message(self) -> Optional[dict]:
        """Receive message"""
        if not self.connected or self._websocket is None:
            raise ConnectionClosedException("Connection is closed")

        try:
            message = await self._websocket.receive()
        except RuntimeError:
            return None

        if message.type == aiohttp.WSMsgType.ERROR:
            raise ConnectionErrorException(self._websocket.exception())

        if message.type in (
            aiohttp.WSMsgType.CLOSE,
            aiohttp.WSMsgType.CLOSED,
            aiohttp.WSMsgType.CLOSING,
        ):
            raise ConnectionClosedException("Connection closed to server")

        if message.type == aiohttp.WSMsgType.TEXT:
            message_json = message.json()

            if (
                message_json[EVENT_TYPE] == TYPE_ERROR
                and message_json[EVENT_SUBTYPE] == SUBTYPE_BAD_API_KEY
            ):
                raise AuthenticationException(message_json[EVENT_MESSAGE])

            return message_json

        raise BadMessageException(f"Unknown message type: {message.type}")



# generated by datamodel-codegen:
#   filename:  request.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class Request(BaseModel):
    """
    Request
    """

    class Config:
        extra = Extra.allow

    api_key: Optional[str] = Field(None, alias="api-key", description="API Key")
    id: Optional[str] = Field(None, description="Message ID")
    event: str = Field(..., description="Event")



"""System Bridge Shared: Logger"""
import logging
from logging.handlers import RotatingFileHandler
import os

from colorlog import ColoredFormatter

from .common import get_user_data_directory
from .const import DATE_FORMAT, FORMAT


def setup_logger(
    log_level: str,
    name: str,
) -> logging.Logger:
    """Set up logging"""

    logging.basicConfig(
        datefmt=DATE_FORMAT,
        format=FORMAT,
        level=log_level,
    )

    logging.getLogger().handlers[0].setFormatter(
        ColoredFormatter(
            f"%(log_color)s{FORMAT}%(reset)s",
            datefmt=DATE_FORMAT,
            reset=True,
            log_colors={
                "DEBUG": "cyan",
                "INFO": "green",
                "WARNING": "yellow",
                "ERROR": "red",
                "CRITICAL": "red",
            },
        )
    )

    file_handler = RotatingFileHandler(
        os.path.join(get_user_data_directory(), f"{name}.log"),
        backupCount=1,
    )
    try:
        file_handler.doRollover()
    except PermissionError:
        pass
    file_handler.setLevel(log_level)
    file_handler.setFormatter(logging.Formatter(FORMAT, datefmt=DATE_FORMAT))

    logger = logging.getLogger("")
    logger.addHandler(file_handler)
    logger.setLevel(log_level)

    return logger



"""System Bridge: Models - Database Data Remote Bridge"""

from __future__ import annotations

from typing import Optional

from sqlmodel import Field, SQLModel


class RemoteBridge(SQLModel, table=True):
    """Database Data Remote Bridge"""

    key: str = Field(primary_key=True, nullable=False)
    name: str = Field(nullable=False)
    host: str = Field(default=None, nullable=False)
    port: str = Field(default=9170, nullable=False)
    api_key: str = Field(nullable=False)
    timestamp: Optional[float] = Field(default=None, nullable=True)



# generated by datamodel-codegen:
#   filename:  memory.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    swap_total: Optional[float] = None
    swap_used: Optional[float] = None
    swap_free: Optional[float] = None
    swap_percent: Optional[float] = None
    swap_sin: Optional[float] = None
    swap_sout: Optional[float] = None
    virtual_total: Optional[float] = None
    virtual_available: Optional[float] = None
    virtual_percent: Optional[float] = None
    virtual_used: Optional[float] = None
    virtual_free: Optional[float] = None


class Memory(BaseModel):
    """
    Memory
    """

    class Config:
        extra = Extra.allow

    id: Optional[str] = Field(None, description="Event ID")
    swap_total: Optional[int] = None
    swap_used: Optional[int] = None
    swap_free: Optional[float] = None
    swap_percent: Optional[float] = None
    swap_sin: Optional[int] = None
    swap_sout: Optional[int] = None
    virtual_total: Optional[int] = None
    virtual_available: Optional[int] = None
    virtual_percent: Optional[float] = None
    virtual_used: Optional[int] = None
    virtual_free: Optional[int] = None
    last_updated: Optional[LastUpdated] = Field(None, description="Last updated")



# generated by datamodel-codegen:
#   filename:  battery.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    is_charging: Optional[float] = None
    percentage: Optional[float] = None


class Battery(BaseModel):
    """
    Battery
    """

    class Config:
        extra = Extra.allow

    id: Optional[str] = Field(None, description="Event ID")
    is_charging: Optional[bool] = None
    percentage: Optional[float] = None
    last_updated: Optional[LastUpdated] = Field(None, description="Last updated")



"""System Bridge Shared: Base"""
import logging


class Base:  # pylint: disable=too-few-public-methods
    """Base"""

    def __init__(self):
        """Initialize"""
        name = f"{self.__module__}.{self.__class__.__name__}"
        self._logger = logging.getLogger(name)
        self._logger.debug("%s __init__", name)



"""Setup"""
import io

from setuptools import find_packages, setup

# Get setup packages from requirements.txt
with io.open("requirements_setup.txt", encoding="utf-8") as f:
    requirements_setup = f.read().splitlines()

# Get packages from requirements.txt
with io.open("requirements.txt", encoding="utf-8") as f:
    requirements = f.read().splitlines()

setup(
    name="systembridgeshared",
    description="System Bridge Shared",
    keywords="system bridge shared",
    author="Aidan Timson (Timmo)",
    author_email="contact@timmo.xyz",
    license="MIT",
    url="https://github.com/timmo001/system-bridge",
    packages=find_packages(exclude=["tests", "generator"]),
    install_requires=requirements,
    setup_requires=requirements_setup,
    use_incremental=True,
)



"""System Bridge Shared: Database"""
from __future__ import annotations

import os
from collections.abc import Mapping
from time import time
from typing import Any, Optional, Union

from sqlmodel import Session, SQLModel, create_engine, select
from sqlmodel.sql.expression import Select, SelectOfScalar

from .base import Base
from .common import convert_string_to_correct_type, get_user_data_directory
from .const import (
    MODEL_BATTERY,
    MODEL_CPU,
    MODEL_DISK,
    MODEL_DISPLAY,
    MODEL_GPU,
    MODEL_MEMORY,
    MODEL_NETWORK,
    MODEL_SECRETS,
    MODEL_SENSORS,
    MODEL_SETTINGS,
    MODEL_SYSTEM,
)
from .models.data import DataDict
from .models.database_data import (
    CPU,
    GPU,
    Battery,
    Data,
    Disk,
    Display,
    Memory,
    Network,
    Secrets,
    Settings,
    System,
)
from .models.database_data_remote_bridge import RemoteBridge
from .models.database_data_sensors import Sensors

TABLE_MAP: Mapping[str, Any] = {
    MODEL_BATTERY: Battery,
    MODEL_CPU: CPU,
    MODEL_DISK: Disk,
    MODEL_DISPLAY: Display,
    MODEL_GPU: GPU,
    MODEL_MEMORY: Memory,
    MODEL_NETWORK: Network,
    MODEL_SECRETS: Secrets,
    MODEL_SENSORS: Sensors,
    MODEL_SETTINGS: Settings,
    MODEL_SYSTEM: System,
}


TableDataType = Union[
    Battery,
    CPU,
    Disk,
    Display,
    GPU,
    Memory,
    Network,
    RemoteBridge,
    Secrets,
    Sensors,
    Settings,
    System,
]


SelectOfScalar.inherit_cache = True  # type: ignore
Select.inherit_cache = True  # type: ignore


class Database(Base):
    """Database"""

    def __init__(self):
        """Initialise"""
        super().__init__()
        self._engine = create_engine(
            f"sqlite:///{os.path.join(get_user_data_directory(), 'systembridge.db')}"
        )
        SQLModel.metadata.create_all(
            self._engine,
            # tables=TABLES,
        )

    def clear_table(
        self,
        table: Any,
    ) -> None:
        """Clear table"""
        with Session(self._engine, autoflush=True) as session:
            for sensor in session.exec(select(table)).all():
                session.delete(sensor)
            session.commit()

    def get_data(
        self,
        table: Any,
    ) -> list[Any]:
        """Get data from database"""
        with Session(self._engine, autoflush=True) as session:
            return session.exec(select(table)).all()

    def get_data_by_key(
        self,
        table: Any,
        key: str,
    ) -> list[Data]:
        """Get data from database by key"""
        with Session(self._engine, autoflush=True) as session:
            return session.exec(select(table).where(table.key == key)).all()

    def get_data_item_by_key(
        self,
        table: Any,
        key: str,
    ) -> Optional[Data]:
        """Get data item from database by key"""
        with Session(self._engine, autoflush=True) as session:
            return session.exec(select(table).where(table.key == key)).first()

    def get_data_dict(
        self,
        table: Any,
    ) -> DataDict:
        """Get data from database as dictionary"""
        data: dict[str, Any] = {}
        data_last_updated: dict[str, float | None] = {}
        result = self.get_data(table)
        for item in result:
            if item.value is None:
                data[item.key] = None
            else:
                data[item.key] = convert_string_to_correct_type(item.value)
            if item.timestamp is None:
                data_last_updated[item.key] = None
            else:
                data_last_updated[item.key] = item.timestamp

        return DataDict(**data, last_updated=data_last_updated)

    def update_data(
        self,
        table,
        data: Any,
    ) -> None:
        """Update data"""
        with Session(self._engine, autoflush=True) as session:
            result = session.exec(select(table).where(table.key == data.key))
            if (old_data := result.first()) is None:
                data.timestamp = time()
                session.add(data)
            else:
                old_data.value = data.value
                old_data.timestamp = time()
                session.add(old_data)
            session.commit()
            if old_data is not None:
                session.refresh(old_data)

    def delete_remote_bridge(
        self,
        key: str,
    ) -> None:
        """Delete remote bridge"""
        with Session(self._engine, autoflush=True) as session:
            result = session.exec(select(RemoteBridge).where(RemoteBridge.key == key))
            if (data := result.first()) is not None:
                session.delete(data)
                session.commit()

    def update_remote_bridge(
        self,
        data: RemoteBridge,
    ) -> RemoteBridge:
        """Update remote bridge"""
        with Session(self._engine, autoflush=True) as session:
            result = session.exec(
                select(RemoteBridge).where(RemoteBridge.key == data.key)
            )
            if (old_data := result.first()) is None:
                data.timestamp = time()
                session.add(data)
            else:
                old_data.name = data.name
                old_data.host = data.host
                old_data.port = data.port
                old_data.api_key = data.api_key
                old_data.timestamp = time()
                session.add(old_data)
            session.commit()
            if old_data is not None:
                session.refresh(old_data)
                return old_data
            return data



"""System Bridge: Models - Database Data"""

from __future__ import annotations

from typing import Optional

from sqlmodel import Field, SQLModel


class Data(SQLModel):
    """Database Data"""

    key: str = Field(primary_key=True, nullable=False)
    value: Optional[str] = Field(default=None, nullable=True)
    timestamp: Optional[float] = Field(default=None, nullable=True)


class Battery(Data, table=True):
    """Database Data Battery"""


class CPU(Data, table=True):
    """Database Data CPU"""


class Disk(Data, table=True):
    """Database Data Disk"""


class Display(Data, table=True):
    """Database Data Display"""


class GPU(Data, table=True):
    """Database Data GPU"""


class Memory(Data, table=True):
    """Database Data Memory"""


class Network(Data, table=True):
    """Database Data Network"""


class Secrets(Data, table=True):
    """Database Data Secrets"""


class Settings(Data, table=True):
    """Database Data Settings"""


class System(Data, table=True):
    """System"""



# generated by datamodel-codegen:
#   filename:  get_setting.json

from __future__ import annotations

from pydantic import BaseModel


class GetSetting(BaseModel):
    """
    Get Setting
    """

    setting: str



"""System Bridge Shared: Constants"""
from .models.battery import Battery
from .models.cpu import Cpu
from .models.data import Data
from .models.disk import Disk
from .models.display import Display
from .models.generic import Generic
from .models.gpu import Gpu
from .models.keyboard_key import KeyboardKey
from .models.keyboard_text import KeyboardText
from .models.media_directories import MediaDirectories
from .models.media_files import File, MediaFiles
from .models.memory import Memory
from .models.network import Network
from .models.notification import Notification
from .models.open_path import OpenPath
from .models.open_url import OpenUrl
from .models.response import Response
from .models.sensors import Sensors
from .models.system import System

# Logging
DATE_FORMAT = "%Y-%m-%d %H:%M:%S"
FORMAT = "%(asctime)s %(levelname)s (%(threadName)s) [%(name)s] %(message)s"

# Database
TABLE_SECRETS = "secrets"
TABLE_SETTINGS = "settings"

COLUMN_HARDWARE_NAME = "hardware_name"
COLUMN_HARDWARE_TYPE = "hardware_type"
COLUMN_KEY = "key"
COLUMN_NAME = "name"
COLUMN_TIMESTAMP = "timestamp"
COLUMN_TYPE = "type"
COLUMN_VALUE = "value"

# Settings
SETTING_ADDITIONAL_MEDIA_DIRECTORIES = "additional_media_directories"
SETTING_AUTOSTART = "autostart"
SETTING_KEYBOARD_HOTKEYS = "keyboard_hotkeys"
SETTING_LOG_LEVEL = "log_level"
SETTING_PORT_API = "port_api"

# Secrets
SECRET_API_KEY = "api_key"

# Headers
HEADER_API_KEY = "api-key"

# Query Parameters
QUERY_ALBUM = "album"
QUERY_API_KEY = "apiKey"
QUERY_API_PORT = "apiPort"
QUERY_ARTIST = "artist"
QUERY_AUTOPLAY = "autoplay"
QUERY_BASE = "base"
QUERY_FILENAME = "filename"
QUERY_PATH = "path"
QUERY_TITLE = "title"
QUERY_URL = "url"
QUERY_VOLUME = "volume"

# Event Keys
EVENT_API_KEY = "api-key"
EVENT_APP_ICON = "app_icon"
EVENT_APP_NAME = "app_name"
EVENT_BASE = "base"
EVENT_DATA = "data"
EVENT_DIRECTORIES = "directories"
EVENT_EVENT = "event"
EVENT_FILE = "file"
EVENT_FILENAME = "filename"
EVENT_FILES = "files"
EVENT_ID = "id"
EVENT_KEY = "key"
EVENT_MESSAGE = "message"
EVENT_MODULE = "module"
EVENT_MODULES = "modules"
EVENT_PATH = "path"
EVENT_SETTING = "setting"
EVENT_SUBTYPE = "subtype"
EVENT_TEXT = "text"
EVENT_TIMEOUT = "timeout"
EVENT_TITLE = "title"
EVENT_TYPE = "type"
EVENT_URL = "url"
EVENT_VALUE = "value"
EVENT_VERSION = "version"
EVENT_VERSIONS = "versions"

# Event Types
SUBTYPE_BAD_API_KEY = "BAD_API_KEY"
SUBTYPE_BAD_DIRECTORY = "BAD_DIRECTORY"
SUBTYPE_BAD_FILE = "BAD_FILE"
SUBTYPE_BAD_JSON = "BAD_JSON"
SUBTYPE_BAD_KEY = "MISSING_KEY"
SUBTYPE_BAD_PATH = "BAD_PATH"
SUBTYPE_BAD_REQUEST = "BAD_REQUEST"
SUBTYPE_LISTENER_ALREADY_REGISTERED = "LISTENER_ALREADY_REGISTERED"
SUBTYPE_LISTENER_NOT_REGISTERED = "LISTENER_NOT_REGISTERED"
SUBTYPE_MISSING_API_KEY = "MISSING_API_KEY"
SUBTYPE_MISSING_BASE = "MISSING_BASE"
SUBTYPE_MISSING_KEY = "MISSING_KEY"
SUBTYPE_MISSING_TITLE = "MISSING_TITLE"
SUBTYPE_MISSING_MODULES = "MISSING_MODULES"
SUBTYPE_MISSING_PATH = "MISSING_PATH"
SUBTYPE_MISSING_PATH_URL = "MISSING_PATH_URL"
SUBTYPE_MISSING_SETTING = "MISSING_SETTING"
SUBTYPE_MISSING_TEXT = "MISSING_TEXT"
SUBTYPE_MISSING_VALUE = "MISSING_VALUE"
SUBTYPE_UNKNOWN_EVENT = "UNKNOWN_EVENT"
TYPE_APPLICATION_UPDATE = "APPLICATION_UPDATE"
TYPE_APPLICATION_UPDATING = "APPLICATION_UPDATING"
TYPE_DATA_GET = "DATA_GET"
TYPE_DATA_LISTENER_REGISTERED = "DATA_LISTENER_REGISTERED"
TYPE_DATA_LISTENER_UNREGISTERED = "DATA_LISTENER_UNREGISTERED"
TYPE_DATA_UPDATE = "DATA_UPDATE"
TYPE_DIRECTORIES = "DIRECTORIES"
TYPE_ERROR = "ERROR"
TYPE_EXIT_APPLICATION = "EXIT_APPLICATION"
TYPE_FILE = "FILE"
TYPE_FILES = "FILES"
TYPE_GET_DATA = "GET_DATA"
TYPE_GET_DIRECTORIES = "GET_DIRECTORIES"
TYPE_GET_FILE = "GET_FILE"
TYPE_GET_FILES = "GET_FILES"
TYPE_GET_REMOTE_BRIDGES = "GET_REMOTE_BRIDGES"
TYPE_GET_REMOTE_BRIDGES_RESULT = "GET_REMOTE_BRIDGES_RESULT"
TYPE_GET_SETTING = "GET_SETTING"
TYPE_GET_SETTINGS = "GET_SETTINGS"
TYPE_KEYBOARD_KEY_PRESSED = "KEYBOARD_KEY_PRESSED"
TYPE_KEYBOARD_KEYPRESS = "KEYBOARD_KEYPRESS"
TYPE_KEYBOARD_TEXT = "KEYBOARD_TEXT"
TYPE_KEYBOARD_TEXT_SENT = "KEYBOARD_TEXT_SENT"
TYPE_NOTIFICATION = "NOTIFICATION"
TYPE_NOTIFICATION_SENT = "NOTIFICATION_SENT"
TYPE_OPEN = "OPEN"
TYPE_OPENED = "OPENED"
TYPE_POWER_HIBERNATE = "POWER_HIBERNATE"
TYPE_POWER_HIBERNATING = "POWER_HIBERNATING"
TYPE_POWER_LOCK = "POWER_LOCK"
TYPE_POWER_LOCKING = "POWER_LOCKING"
TYPE_POWER_LOGGINGOUT = "POWER_LOGGINGOUT"
TYPE_POWER_LOGOUT = "POWER_LOGOUT"
TYPE_POWER_RESTART = "POWER_RESTART"
TYPE_POWER_RESTARTING = "POWER_RESTARTING"
TYPE_POWER_SHUTDOWN = "POWER_SHUTDOWN"
TYPE_POWER_SHUTTINGDOWN = "POWER_SHUTTINGDOWN"
TYPE_POWER_SLEEP = "POWER_SLEEP"
TYPE_POWER_SLEEPING = "POWER_SLEEPING"
TYPE_REGISTER_DATA_LISTENER = "REGISTER_DATA_LISTENER"
TYPE_SETTING_RESULT = "SETTING_RESULT"
TYPE_SETTING_UPDATED = "SETTING_UPDATED"
TYPE_SETTINGS_RESULT = "SETTINGS_RESULT"
TYPE_UNREGISTER_DATA_LISTENER = "UNREGISTER_DATA_LISTENER"
TYPE_UPDATE_REMOTE_BRIDGE = "UPDATE_REMOTE_BRIDGE"
TYPE_UPDATE_REMOTE_BRIDGE_RESULT = "UPDATE_REMOTE_BRIDGE_RESULT"
TYPE_UPDATE_SETTING = "UPDATE_SETTING"

# Model
MODEL_BATTERY = "battery"
MODEL_CPU = "cpu"
MODEL_DATA = "data"
MODEL_DISK = "disk"
MODEL_DISPLAY = "display"
MODEL_GENERIC = "generic"
MODEL_GPU = "gpu"
MODEL_KEYBOARD_KEY = "keyboard_key"
MODEL_KEYBOARD_TEXT = "keyboard_text"
MODEL_MEDIA_DIRECTORIES = "media_directories"
MODEL_MEDIA_FILE = "media_file"
MODEL_MEDIA_FILES = "media_files"
MODEL_MEMORY = "memory"
MODEL_NETWORK = "network"
MODEL_NOTIFICATION = "notification"
MODEL_OPEN_PATH = "open_path"
MODEL_OPEN_URL = "open_url"
MODEL_RESPONSE = "response"
MODEL_SECRETS = "secrets"
MODEL_SENSORS = "sensors"
MODEL_SETTINGS = "settings"
MODEL_SYSTEM = "system"

MODEL_MAP = {
    MODEL_BATTERY: Battery,
    MODEL_CPU: Cpu,
    MODEL_DATA: Data,
    MODEL_DISK: Disk,
    MODEL_DISPLAY: Display,
    MODEL_GENERIC: Generic,
    MODEL_GPU: Gpu,
    MODEL_KEYBOARD_KEY: KeyboardKey,
    MODEL_KEYBOARD_TEXT: KeyboardText,
    MODEL_MEDIA_DIRECTORIES: MediaDirectories,
    MODEL_MEDIA_FILE: File,
    MODEL_MEDIA_FILES: MediaFiles,
    MODEL_MEMORY: Memory,
    MODEL_NETWORK: Network,
    MODEL_NOTIFICATION: Notification,
    MODEL_OPEN_PATH: OpenPath,
    MODEL_OPEN_URL: OpenUrl,
    MODEL_RESPONSE: Response,
    MODEL_SENSORS: Sensors,
    MODEL_SYSTEM: System,
}



"""System Bridge: Models - Database Data Sensors"""

from __future__ import annotations

from typing import Optional

from sqlmodel import Field

from .database_data import Data


class Sensors(Data, table=True):
    """Database Data Sensors"""

    type: str = Field(nullable=False)
    name: Optional[str] = Field(default=None, nullable=True)
    hardware_type: Optional[str] = Field(default=None, nullable=True)
    hardware_name: Optional[str] = Field(default=None, nullable=True)
